@@ -1,6 +1,5 @@
 """The aurora component."""
 
-import asyncio
 from datetime import timedelta
 import logging
 
@@ -8,10 +7,11 @@ from aiohttp import ClientError
 from auroranoaa import AuroraForecast
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ATTR_NAME, CONF_LATITUDE, CONF_LONGITUDE, CONF_NAME
+from homeassistant.const import CONF_LATITUDE, CONF_LONGITUDE, CONF_NAME, Platform
 from homeassistant.core import HomeAssistant
-from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.helpers import aiohttp_client
+from homeassistant.helpers.device_registry import DeviceEntryType
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.update_coordinator import (
     CoordinatorEntity,
     DataUpdateCoordinator,
@@ -19,10 +19,6 @@ from homeassistant.helpers.update_coordinator import (
 )
 
 from .const import (
-    ATTR_ENTRY_TYPE,
-    ATTR_IDENTIFIERS,
-    ATTR_MANUFACTURER,
-    ATTR_MODEL,
     ATTRIBUTION,
     AURORA_API,
     CONF_THRESHOLD,
@@ -34,17 +30,10 @@ from .const import (
 
 _LOGGER = logging.getLogger(__name__)
 
-PLATFORMS = ["binary_sensor", "sensor"]
+PLATFORMS = [Platform.BINARY_SENSOR, Platform.SENSOR]
 
 
-async def async_setup(hass: HomeAssistant, config: dict):
-    """Set up the Aurora component."""
-    hass.data.setdefault(DOMAIN, {})
-
-    return True
-
-
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Aurora from a config entry."""
 
     conf = entry.data
@@ -69,34 +58,23 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         threshold=threshold,
     )
 
-    await coordinator.async_refresh()
-
-    if not coordinator.last_update_success:
-        raise ConfigEntryNotReady
+    await coordinator.async_config_entry_first_refresh()
 
+    hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.entry_id] = {
         COORDINATOR: coordinator,
         AURORA_API: api,
     }
 
-    for component in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, component)
-                for component in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
+
     if unload_ok:
         hass.data[DOMAIN].pop(entry.entry_id)
 
@@ -115,7 +93,7 @@ class AuroraDataUpdateCoordinator(DataUpdateCoordinator):
         latitude: float,
         longitude: float,
         threshold: float,
-    ):
+    ) -> None:
         """Initialize the data updater."""
 
         super().__init__(
@@ -140,50 +118,32 @@ class AuroraDataUpdateCoordinator(DataUpdateCoordinator):
             raise UpdateFailed(f"Error updating from NOAA: {error}") from error
 
 
-class AuroraEntity(CoordinatorEntity):
+class AuroraEntity(CoordinatorEntity[AuroraDataUpdateCoordinator]):
     """Implementation of the base Aurora Entity."""
 
+    _attr_extra_state_attributes = {"attribution": ATTRIBUTION}
+
     def __init__(
         self,
         coordinator: AuroraDataUpdateCoordinator,
         name: str,
         icon: str,
-    ):
+    ) -> None:
         """Initialize the Aurora Entity."""
 
         super().__init__(coordinator=coordinator)
 
-        self._name = name
-        self._unique_id = f"{self.coordinator.latitude}_{self.coordinator.longitude}"
-        self._icon = icon
+        self._attr_name = name
+        self._attr_unique_id = f"{coordinator.latitude}_{coordinator.longitude}"
+        self._attr_icon = icon
 
     @property
-    def unique_id(self):
-        """Define the unique id based on the latitude and longitude."""
-        return self._unique_id
-
-    @property
-    def name(self):
-        """Return the name of the sensor."""
-        return self._name
-
-    @property
-    def device_state_attributes(self):
-        """Return the state attributes."""
-        return {"attribution": ATTRIBUTION}
-
-    @property
-    def icon(self):
-        """Return the icon for the sensor."""
-        return self._icon
-
-    @property
-    def device_info(self):
+    def device_info(self) -> DeviceInfo:
         """Define the device based on name."""
-        return {
-            ATTR_IDENTIFIERS: {(DOMAIN, self._unique_id)},
-            ATTR_NAME: self.coordinator.name,
-            ATTR_MANUFACTURER: "NOAA",
-            ATTR_MODEL: "Aurora Visibility Sensor",
-            ATTR_ENTRY_TYPE: "service",
-        }
+        return DeviceInfo(
+            entry_type=DeviceEntryType.SERVICE,
+            identifiers={(DOMAIN, str(self.unique_id))},
+            manufacturer="NOAA",
+            model="Aurora Visibility Sensor",
+            name=self.coordinator.name,
+        )
