@@ -1,7 +1,10 @@
 """Support for Bluesound devices."""
+from __future__ import annotations
+
 import asyncio
 from asyncio import CancelledError
 from datetime import timedelta
+from http import HTTPStatus
 import logging
 from urllib import parse
 
@@ -12,10 +15,15 @@ import async_timeout
 import voluptuous as vol
 import xmltodict
 
-from homeassistant.components.media_player import PLATFORM_SCHEMA, MediaPlayerDevice
+from homeassistant.components import media_source
+from homeassistant.components.media_player import PLATFORM_SCHEMA, MediaPlayerEntity
+from homeassistant.components.media_player.browse_media import (
+    async_process_play_media_url,
+)
 from homeassistant.components.media_player.const import (
     ATTR_MEDIA_ENQUEUE,
     MEDIA_TYPE_MUSIC,
+    SUPPORT_BROWSE_MEDIA,
     SUPPORT_CLEAR_PLAYLIST,
     SUPPORT_NEXT_TRACK,
     SUPPORT_PAUSE,
@@ -43,10 +51,12 @@ from homeassistant.const import (
     STATE_PAUSED,
     STATE_PLAYING,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_track_time_interval
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util import Throttle
 import homeassistant.util.dt as dt_util
 
@@ -105,8 +115,6 @@ SERVICE_TO_METHOD = {
 
 def _add_player(hass, async_add_entities, host, port=None, name=None):
     """Add Bluesound players."""
-    if host in [x.host for x in hass.data[DATA_BLUESOUND]]:
-        return
 
     @callback
     def _init_player(event=None):
@@ -126,6 +134,11 @@ def _add_player(hass, async_add_entities, host, port=None, name=None):
     @callback
     def _add_player_cb():
         """Add player after first sync fetch."""
+        if player.id in [x.id for x in hass.data[DATA_BLUESOUND]]:
+            _LOGGER.warning("Player already added %s", player.id)
+            return
+
+        hass.data[DATA_BLUESOUND].append(player)
         async_add_entities([player])
         _LOGGER.info("Added device with name: %s", player.name)
 
@@ -137,7 +150,6 @@ def _add_player(hass, async_add_entities, host, port=None, name=None):
     hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, _stop_polling)
 
     player = BluesoundPlayer(hass, host, port, name, _add_player_cb)
-    hass.data[DATA_BLUESOUND].append(player)
 
     if hass.is_running:
         _init_player()
@@ -145,7 +157,12 @@ def _add_player(hass, async_add_entities, host, port=None, name=None):
         hass.bus.async_listen_once(EVENT_HOMEASSISTANT_START, _init_player)
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+async def async_setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the Bluesound platforms."""
     if DATA_BLUESOUND not in hass.data:
         hass.data[DATA_BLUESOUND] = []
@@ -155,12 +172,11 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
             hass,
             async_add_entities,
             discovery_info.get(CONF_HOST),
-            discovery_info.get(CONF_PORT, None),
+            discovery_info.get(CONF_PORT),
         )
         return
 
-    hosts = config.get(CONF_HOSTS, None)
-    if hosts:
+    if hosts := config.get(CONF_HOSTS):
         for host in hosts:
             _add_player(
                 hass,
@@ -170,17 +186,15 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
                 host.get(CONF_NAME),
             )
 
-    async def async_service_handler(service):
+    async def async_service_handler(service: ServiceCall) -> None:
         """Map services to method of Bluesound devices."""
-        method = SERVICE_TO_METHOD.get(service.service)
-        if not method:
+        if not (method := SERVICE_TO_METHOD.get(service.service)):
             return
 
         params = {
             key: value for key, value in service.data.items() if key != ATTR_ENTITY_ID
         }
-        entity_ids = service.data.get(ATTR_ENTITY_ID)
-        if entity_ids:
+        if entity_ids := service.data.get(ATTR_ENTITY_ID):
             target_players = [
                 player
                 for player in hass.data[DATA_BLUESOUND]
@@ -192,14 +206,14 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
         for player in target_players:
             await getattr(player, method["method"])(**params)
 
-    for service in SERVICE_TO_METHOD:
-        schema = SERVICE_TO_METHOD[service]["schema"]
+    for service, method in SERVICE_TO_METHOD.items():
+        schema = method["schema"]
         hass.services.async_register(
             DOMAIN, service, async_service_handler, schema=schema
         )
 
 
-class BluesoundPlayer(MediaPlayerDevice):
+class BluesoundPlayer(MediaPlayerEntity):
     """Representation of a Bluesound Player."""
 
     def __init__(self, hass, host, port=None, name=None, init_callback=None):
@@ -210,6 +224,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         self._polling_session = async_get_clientsession(hass)
         self._polling_task = None  # The actual polling task.
         self._name = name
+        self._id = None
         self._icon = None
         self._capture_items = []
         self._services_items = []
@@ -219,7 +234,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         self._last_status_update = None
         self._is_online = False
         self._retry_remove = None
-        self._lastvol = None
+        self._muted = False
         self._master = None
         self._is_master = False
         self._group_name = None
@@ -227,6 +242,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         self._bluesound_device_name = None
 
         self._init_callback = init_callback
+
         if self.port is None:
             self.port = DEFAULT_PORT
 
@@ -253,30 +269,33 @@ class BluesoundPlayer(MediaPlayerDevice):
 
         if not self._name:
             self._name = self._sync_status.get("@name", self.host)
+        if not self._id:
+            self._id = self._sync_status.get("@id", None)
         if not self._bluesound_device_name:
             self._bluesound_device_name = self._sync_status.get("@name", self.host)
         if not self._icon:
             self._icon = self._sync_status.get("@icon", self.host)
 
-        master = self._sync_status.get("master", None)
-        if master is not None:
+        if (master := self._sync_status.get("master")) is not None:
             self._is_master = False
             master_host = master.get("#text")
+            master_port = master.get("@port", "11000")
+            master_id = f"{master_host}:{master_port}"
             master_device = [
                 device
                 for device in self._hass.data[DATA_BLUESOUND]
-                if device.host == master_host
+                if device.id == master_id
             ]
 
-            if master_device and master_host != self.host:
+            if master_device and master_id != self.id:
                 self._master = master_device[0]
             else:
                 self._master = None
-                _LOGGER.error("Master not found %s", master_host)
+                _LOGGER.error("Master not found %s", master_id)
         else:
             if self._master is not None:
                 self._master = None
-            slaves = self._sync_status.get("slave", None)
+            slaves = self._sync_status.get("slave")
             self._is_master = slaves is not None
 
         if on_updated_cb:
@@ -290,14 +309,14 @@ class BluesoundPlayer(MediaPlayerDevice):
                 await self.async_update_status()
 
         except (asyncio.TimeoutError, ClientError, BluesoundPlayer._TimeoutException):
-            _LOGGER.info("Node %s is offline, retrying later", self._name)
+            _LOGGER.info("Node %s:%s is offline, retrying later", self.name, self.port)
             await asyncio.sleep(NODE_OFFLINE_CHECK_TIMEOUT)
             self.start_polling()
 
         except CancelledError:
-            _LOGGER.debug("Stopping the polling of node %s", self._name)
+            _LOGGER.debug("Stopping the polling of node %s:%s", self.name, self.port)
         except Exception:
-            _LOGGER.exception("Unexpected error in %s", self._name)
+            _LOGGER.exception("Unexpected error in %s:%s", self.name, self.port)
             raise
 
     def start_polling(self):
@@ -317,12 +336,14 @@ class BluesoundPlayer(MediaPlayerDevice):
 
             await self.force_update_sync_status(self._init_callback, True)
         except (asyncio.TimeoutError, ClientError):
-            _LOGGER.info("Node %s is offline, retrying later", self.host)
+            _LOGGER.info("Node %s:%s is offline, retrying later", self.host, self.port)
             self._retry_remove = async_track_time_interval(
                 self._hass, self.async_init, NODE_RETRY_INITIATION
             )
         except Exception:
-            _LOGGER.exception("Unexpected when initiating error in %s", self.host)
+            _LOGGER.exception(
+                "Unexpected when initiating error in %s:%s", self.host, self.port
+            )
             raise
 
     async def async_update(self):
@@ -351,10 +372,10 @@ class BluesoundPlayer(MediaPlayerDevice):
 
         try:
             websession = async_get_clientsession(self._hass)
-            with async_timeout.timeout(10):
+            async with async_timeout.timeout(10):
                 response = await websession.get(url)
 
-            if response.status == 200:
+            if response.status == HTTPStatus.OK:
                 result = await response.text()
                 if result:
                     data = xmltodict.parse(result)
@@ -369,9 +390,9 @@ class BluesoundPlayer(MediaPlayerDevice):
 
         except (asyncio.TimeoutError, aiohttp.ClientError):
             if raise_timeout:
-                _LOGGER.info("Timeout: %s", self.host)
+                _LOGGER.info("Timeout: %s:%s", self.host, self.port)
                 raise
-            _LOGGER.debug("Failed communicating: %s", self.host)
+            _LOGGER.debug("Failed communicating: %s:%s", self.host, self.port)
             return None
 
         return data
@@ -393,20 +414,20 @@ class BluesoundPlayer(MediaPlayerDevice):
 
         try:
 
-            with async_timeout.timeout(125):
+            async with async_timeout.timeout(125):
                 response = await self._polling_session.get(
                     url, headers={CONNECTION: KEEP_ALIVE}
                 )
 
-            if response.status == 200:
+            if response.status == HTTPStatus.OK:
                 result = await response.text()
                 self._is_online = True
                 self._last_status_update = dt_util.utcnow()
                 self._status = xmltodict.parse(result)["status"].copy()
 
-                group_name = self._status.get("groupName", None)
+                group_name = self._status.get("groupName")
                 if group_name != self._group_name:
-                    _LOGGER.debug("Group name change detected on device: %s", self.host)
+                    _LOGGER.debug("Group name change detected on device: %s", self.id)
                     self._group_name = group_name
 
                     # rebuild ordered list of entity_ids that are in the group, master is first
@@ -426,7 +447,7 @@ class BluesoundPlayer(MediaPlayerDevice):
                     # communication is moved to a separate library
                     await self.force_update_sync_status()
 
-                self.async_schedule_update_ha_state()
+                self.async_write_ha_state()
             elif response.status == 595:
                 _LOGGER.info("Status 595 returned, treating as timeout")
                 raise BluesoundPlayer._TimeoutException()
@@ -439,7 +460,7 @@ class BluesoundPlayer(MediaPlayerDevice):
             self._is_online = False
             self._last_status_update = None
             self._status = None
-            self.async_schedule_update_ha_state()
+            self.async_write_ha_state()
             _LOGGER.info("Client connection error, marking %s as offline", self._name)
             raise
 
@@ -555,7 +576,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self.is_grouped and not self.is_master:
             return STATE_GROUPED
 
-        status = self._status.get("state", None)
+        status = self._status.get("state")
         if status in ("pause", "stop"):
             return STATE_PAUSED
         if status in ("stream", "play"):
@@ -568,7 +589,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._status is None or (self.is_grouped and not self.is_master):
             return None
 
-        return self._status.get("title1", None)
+        return self._status.get("title1")
 
     @property
     def media_artist(self):
@@ -579,9 +600,8 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self.is_grouped and not self.is_master:
             return self._group_name
 
-        artist = self._status.get("artist", None)
-        if not artist:
-            artist = self._status.get("title2", None)
+        if not (artist := self._status.get("artist")):
+            artist = self._status.get("title2")
         return artist
 
     @property
@@ -590,9 +610,8 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._status is None or (self.is_grouped and not self.is_master):
             return None
 
-        album = self._status.get("album", None)
-        if not album:
-            album = self._status.get("title3", None)
+        if not (album := self._status.get("album")):
+            album = self._status.get("title3")
         return album
 
     @property
@@ -601,8 +620,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._status is None or (self.is_grouped and not self.is_master):
             return None
 
-        url = self._status.get("image", None)
-        if not url:
+        if not (url := self._status.get("image")):
             return
         if url[0] == "/":
             url = f"http://{self.host}:{self.port}{url}"
@@ -619,8 +637,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._last_status_update is None or mediastate == STATE_IDLE:
             return None
 
-        position = self._status.get("secs", None)
-        if position is None:
+        if (position := self._status.get("secs")) is None:
             return None
 
         position = float(position)
@@ -635,8 +652,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._status is None or (self.is_grouped and not self.is_master):
             return None
 
-        duration = self._status.get("totlen", None)
-        if duration is None:
+        if (duration := self._status.get("totlen")) is None:
             return None
         return float(duration)
 
@@ -648,9 +664,9 @@ class BluesoundPlayer(MediaPlayerDevice):
     @property
     def volume_level(self):
         """Volume level of the media player (0..1)."""
-        volume = self._status.get("volume", None)
+        volume = self._status.get("volume")
         if self.is_grouped:
-            volume = self._sync_status.get("@volume", None)
+            volume = self._sync_status.get("@volume")
 
         if volume is not None:
             return int(volume) / 100
@@ -659,10 +675,18 @@ class BluesoundPlayer(MediaPlayerDevice):
     @property
     def is_volume_muted(self):
         """Boolean if volume is currently muted."""
-        volume = self.volume_level
-        if not volume:
-            return None
-        return 0 <= volume < 0.001
+        mute = self._status.get("mute")
+        if self.is_grouped:
+            mute = self._sync_status.get("@mute")
+
+        if mute is not None:
+            mute = bool(int(mute))
+        return mute
+
+    @property
+    def id(self):
+        """Get id of device."""
+        return self._id
 
     @property
     def name(self):
@@ -708,8 +732,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._status is None or (self.is_grouped and not self.is_master):
             return None
 
-        current_service = self._status.get("service", "")
-        if current_service == "":
+        if (current_service := self._status.get("service", "")) == "":
             return ""
         stream_url = self._status.get("streamUrl", "")
 
@@ -782,7 +805,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self.is_grouped and not self.is_master:
             return SUPPORT_VOLUME_STEP | SUPPORT_VOLUME_SET | SUPPORT_VOLUME_MUTE
 
-        supported = SUPPORT_CLEAR_PLAYLIST
+        supported = SUPPORT_CLEAR_PLAYLIST | SUPPORT_BROWSE_MEDIA
 
         if self._status.get("indexing", "0") == "0":
             supported = (
@@ -837,8 +860,8 @@ class BluesoundPlayer(MediaPlayerDevice):
         if master_device:
             _LOGGER.debug(
                 "Trying to join player: %s to master: %s",
-                self.host,
-                master_device[0].host,
+                self.id,
+                master_device[0].id,
             )
 
             await master_device[0].async_add_slave(self)
@@ -846,7 +869,7 @@ class BluesoundPlayer(MediaPlayerDevice):
             _LOGGER.error("Master not found %s", master_device)
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """List members in group."""
         attributes = {}
         if self._group_list:
@@ -883,7 +906,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self._master is None:
             return
 
-        _LOGGER.debug("Trying to unjoin player: %s", self.host)
+        _LOGGER.debug("Trying to unjoin player: %s", self.id)
         await self._master.async_remove_slave(self)
 
     async def async_add_slave(self, slave_device):
@@ -902,7 +925,7 @@ class BluesoundPlayer(MediaPlayerDevice):
         """Increase sleep time on player."""
         sleep_time = await self.send_bluesound_command("/Sleep")
         if sleep_time is None:
-            _LOGGER.error("Error while increasing sleep time on player: %s", self.host)
+            _LOGGER.error("Error while increasing sleep time on player: %s", self.id)
             return 0
 
         return int(sleep_time.get("sleep", "0"))
@@ -1011,6 +1034,12 @@ class BluesoundPlayer(MediaPlayerDevice):
         if self.is_grouped and not self.is_master:
             return
 
+        if media_source.is_media_source_id(media_id):
+            play_item = await media_source.async_resolve_media(self.hass, media_id)
+            media_id = play_item.url
+
+        media_id = async_process_play_media_url(self.hass, media_id)
+
         url = f"Play?url={media_id}"
 
         if kwargs.get(ATTR_MEDIA_ENQUEUE):
@@ -1038,17 +1067,18 @@ class BluesoundPlayer(MediaPlayerDevice):
             volume = 0
         elif volume > 1:
             volume = 1
-        return await self.send_bluesound_command(
-            "Volume?level=" + str(float(volume) * 100)
-        )
+        return await self.send_bluesound_command(f"Volume?level={float(volume) * 100}")
 
     async def async_mute_volume(self, mute):
         """Send mute command to media player."""
         if mute:
-            volume = self.volume_level
-            if volume > 0:
-                self._lastvol = volume
-            return await self.send_bluesound_command("Volume?level=0")
-        return await self.send_bluesound_command(
-            "Volume?level=" + str(float(self._lastvol) * 100)
+            return await self.send_bluesound_command("Volume?mute=1")
+        return await self.send_bluesound_command("Volume?mute=0")
+
+    async def async_browse_media(self, media_content_type=None, media_content_id=None):
+        """Implement the websocket media browsing helper."""
+        return await media_source.async_browse_media(
+            self.hass,
+            media_content_id,
+            content_filter=lambda item: item.media_content_type.startswith("audio/"),
         )
