@@ -1,15 +1,20 @@
 """Support for the Daikin HVAC."""
+from __future__ import annotations
+
 import logging
 
-from pydaikin import appliance
 import voluptuous as vol
 
-from homeassistant.components.climate import PLATFORM_SCHEMA, ClimateDevice
+from homeassistant.components.climate import PLATFORM_SCHEMA, ClimateEntity
 from homeassistant.components.climate.const import (
     ATTR_FAN_MODE,
     ATTR_HVAC_MODE,
     ATTR_PRESET_MODE,
     ATTR_SWING_MODE,
+    CURRENT_HVAC_COOL,
+    CURRENT_HVAC_HEAT,
+    CURRENT_HVAC_IDLE,
+    CURRENT_HVAC_OFF,
     HVAC_MODE_COOL,
     HVAC_MODE_DRY,
     HVAC_MODE_FAN_ONLY,
@@ -17,14 +22,20 @@ from homeassistant.components.climate.const import (
     HVAC_MODE_HEAT_COOL,
     HVAC_MODE_OFF,
     PRESET_AWAY,
+    PRESET_BOOST,
+    PRESET_ECO,
     PRESET_NONE,
     SUPPORT_FAN_MODE,
     SUPPORT_PRESET_MODE,
     SUPPORT_SWING_MODE,
     SUPPORT_TARGET_TEMPERATURE,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import ATTR_TEMPERATURE, CONF_HOST, CONF_NAME, TEMP_CELSIUS
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 from . import DOMAIN as DAIKIN_DOMAIN
 from .const import (
@@ -59,7 +70,18 @@ DAIKIN_TO_HA_STATE = {
     "off": HVAC_MODE_OFF,
 }
 
-HA_PRESET_TO_DAIKIN = {PRESET_AWAY: "on", PRESET_NONE: "off"}
+HA_STATE_TO_CURRENT_HVAC = {
+    HVAC_MODE_COOL: CURRENT_HVAC_COOL,
+    HVAC_MODE_HEAT: CURRENT_HVAC_HEAT,
+    HVAC_MODE_OFF: CURRENT_HVAC_OFF,
+}
+
+HA_PRESET_TO_DAIKIN = {
+    PRESET_AWAY: "on",
+    PRESET_NONE: "off",
+    PRESET_BOOST: "powerful",
+    PRESET_ECO: "econo",
+}
 
 HA_ATTR_TO_DAIKIN = {
     ATTR_PRESET_MODE: "en_hol",
@@ -71,23 +93,36 @@ HA_ATTR_TO_DAIKIN = {
     ATTR_TARGET_TEMPERATURE: "stemp",
 }
 
+DAIKIN_ATTR_ADVANCED = "adv"
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+
+async def async_setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Old way of setting up the Daikin HVAC platform.
 
     Can only be called when a user accidentally mentions the platform in their
     config. But even in that case it would have been ignored.
     """
-    pass
 
 
-async def async_setup_entry(hass, entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback
+) -> None:
     """Set up Daikin climate based on config_entry."""
     daikin_api = hass.data[DAIKIN_DOMAIN].get(entry.entry_id)
     async_add_entities([DaikinClimate(daikin_api)], update_before_add=True)
 
 
-class DaikinClimate(ClimateDevice):
+def format_target_temperature(target_temperature):
+    """Format target temperature to be sent to the Daikin unit, rounding to nearest half degree."""
+    return str(round(float(target_temperature) * 2, 0) / 2).rstrip("0").rstrip(".")
+
+
+class DaikinClimate(ClimateEntity):
     """Representation of a Daikin HVAC."""
 
     def __init__(self, api):
@@ -97,17 +132,15 @@ class DaikinClimate(ClimateDevice):
         self._list = {
             ATTR_HVAC_MODE: list(HA_STATE_TO_DAIKIN),
             ATTR_FAN_MODE: self._api.device.fan_rate,
-            ATTR_SWING_MODE: list(
-                map(
-                    str.title,
-                    appliance.daikin_values(HA_ATTR_TO_DAIKIN[ATTR_SWING_MODE]),
-                )
-            ),
+            ATTR_SWING_MODE: self._api.device.swing_modes,
         }
 
         self._supported_features = SUPPORT_TARGET_TEMPERATURE
 
-        if self._api.device.support_away_mode:
+        if (
+            self._api.device.support_away_mode
+            or self._api.device.support_advanced_modes
+        ):
             self._supported_features |= SUPPORT_PRESET_MODE
 
         if self._api.device.support_fan_rate:
@@ -120,13 +153,11 @@ class DaikinClimate(ClimateDevice):
         """Set device settings using API."""
         values = {}
 
-        for attr in [ATTR_TEMPERATURE, ATTR_FAN_MODE, ATTR_SWING_MODE, ATTR_HVAC_MODE]:
-            value = settings.get(attr)
-            if value is None:
+        for attr in (ATTR_TEMPERATURE, ATTR_FAN_MODE, ATTR_SWING_MODE, ATTR_HVAC_MODE):
+            if (value := settings.get(attr)) is None:
                 continue
 
-            daikin_attr = HA_ATTR_TO_DAIKIN.get(attr)
-            if daikin_attr is not None:
+            if (daikin_attr := HA_ATTR_TO_DAIKIN.get(attr)) is not None:
                 if attr == ATTR_HVAC_MODE:
                     values[daikin_attr] = HA_STATE_TO_DAIKIN[value]
                 elif value in self._list[attr]:
@@ -137,7 +168,9 @@ class DaikinClimate(ClimateDevice):
             # temperature
             elif attr == ATTR_TEMPERATURE:
                 try:
-                    values[HA_ATTR_TO_DAIKIN[ATTR_TARGET_TEMPERATURE]] = str(int(value))
+                    values[
+                        HA_ATTR_TO_DAIKIN[ATTR_TARGET_TEMPERATURE]
+                    ] = format_target_temperature(value)
                 except ValueError:
                     _LOGGER.error("Invalid temperature %s", value)
 
@@ -157,7 +190,7 @@ class DaikinClimate(ClimateDevice):
     @property
     def unique_id(self):
         """Return a unique ID."""
-        return self._api.mac
+        return self._api.device.mac
 
     @property
     def temperature_unit(self):
@@ -183,6 +216,18 @@ class DaikinClimate(ClimateDevice):
         """Set new target temperature."""
         await self._set(kwargs)
 
+    @property
+    def hvac_action(self):
+        """Return the current state."""
+        ret = HA_STATE_TO_CURRENT_HVAC.get(self.hvac_mode)
+        if (
+            ret in (CURRENT_HVAC_COOL, CURRENT_HVAC_HEAT)
+            and self._api.device.support_compressor_frequency
+            and self._api.device.compressor_frequency == 0
+        ):
+            return CURRENT_HVAC_IDLE
+        return ret
+
     @property
     def hvac_mode(self):
         """Return current operation ie. heat, cool, idle."""
@@ -234,19 +279,51 @@ class DaikinClimate(ClimateDevice):
             == HA_PRESET_TO_DAIKIN[PRESET_AWAY]
         ):
             return PRESET_AWAY
+        if (
+            HA_PRESET_TO_DAIKIN[PRESET_BOOST]
+            in self._api.device.represent(DAIKIN_ATTR_ADVANCED)[1]
+        ):
+            return PRESET_BOOST
+        if (
+            HA_PRESET_TO_DAIKIN[PRESET_ECO]
+            in self._api.device.represent(DAIKIN_ATTR_ADVANCED)[1]
+        ):
+            return PRESET_ECO
         return PRESET_NONE
 
     async def async_set_preset_mode(self, preset_mode):
         """Set preset mode."""
         if preset_mode == PRESET_AWAY:
             await self._api.device.set_holiday(ATTR_STATE_ON)
+        elif preset_mode == PRESET_BOOST:
+            await self._api.device.set_advanced_mode(
+                HA_PRESET_TO_DAIKIN[PRESET_BOOST], ATTR_STATE_ON
+            )
+        elif preset_mode == PRESET_ECO:
+            await self._api.device.set_advanced_mode(
+                HA_PRESET_TO_DAIKIN[PRESET_ECO], ATTR_STATE_ON
+            )
         else:
-            await self._api.device.set_holiday(ATTR_STATE_OFF)
+            if self.preset_mode == PRESET_AWAY:
+                await self._api.device.set_holiday(ATTR_STATE_OFF)
+            elif self.preset_mode == PRESET_BOOST:
+                await self._api.device.set_advanced_mode(
+                    HA_PRESET_TO_DAIKIN[PRESET_BOOST], ATTR_STATE_OFF
+                )
+            elif self.preset_mode == PRESET_ECO:
+                await self._api.device.set_advanced_mode(
+                    HA_PRESET_TO_DAIKIN[PRESET_ECO], ATTR_STATE_OFF
+                )
 
     @property
     def preset_modes(self):
         """List of available preset modes."""
-        return list(HA_PRESET_TO_DAIKIN)
+        ret = [PRESET_NONE]
+        if self._api.device.support_away_mode:
+            ret.append(PRESET_AWAY)
+        if self._api.device.support_advanced_modes:
+            ret += [PRESET_ECO, PRESET_BOOST]
+        return ret
 
     async def async_update(self):
         """Retrieve latest state."""
