@@ -1,7 +1,9 @@
 """Support for Fibaro lights."""
+from __future__ import annotations
+
 import asyncio
+from contextlib import suppress
 from functools import partial
-import logging
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
@@ -11,14 +13,15 @@ from homeassistant.components.light import (
     SUPPORT_BRIGHTNESS,
     SUPPORT_COLOR,
     SUPPORT_WHITE_VALUE,
-    Light,
+    LightEntity,
 )
-from homeassistant.const import CONF_WHITE_VALUE
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 import homeassistant.util.color as color_util
 
-from . import CONF_COLOR, CONF_DIMMING, CONF_RESET_COLOR, FIBARO_DEVICES, FibaroDevice
-
-_LOGGER = logging.getLogger(__name__)
+from . import FIBARO_DEVICES, FibaroDevice
+from .const import DOMAIN
 
 
 def scaleto255(value):
@@ -38,17 +41,22 @@ def scaleto100(value):
     return max(0, min(100, ((value * 100.0) / 255.0)))
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Perform the setup for Fibaro controller devices."""
-    if discovery_info is None:
-        return
-
     async_add_entities(
-        [FibaroLight(device) for device in hass.data[FIBARO_DEVICES]["light"]], True
+        [
+            FibaroLight(device)
+            for device in hass.data[DOMAIN][entry.entry_id][FIBARO_DEVICES]["light"]
+        ],
+        True,
     )
 
 
-class FibaroLight(FibaroDevice, Light):
+class FibaroLight(FibaroDevice, LightEntity):
     """Representation of a Fibaro Light, including dimmable."""
 
     def __init__(self, fibaro_device):
@@ -60,20 +68,34 @@ class FibaroLight(FibaroDevice, Light):
         self._update_lock = asyncio.Lock()
         self._white = 0
 
-        devconf = fibaro_device.device_config
-        self._reset_color = devconf.get(CONF_RESET_COLOR, False)
+        self._reset_color = False
         supports_color = (
-            "color" in fibaro_device.properties and "setColor" in fibaro_device.actions
+            "color" in fibaro_device.properties
+            or "colorComponents" in fibaro_device.properties
+            or "RGB" in fibaro_device.type
+            or "rgb" in fibaro_device.type
+            or "color" in fibaro_device.baseType
+        ) and (
+            "setColor" in fibaro_device.actions
+            or "setColorComponents" in fibaro_device.actions
+        )
+        supports_white_v = (
+            "setW" in fibaro_device.actions
+            or "RGBW" in fibaro_device.type
+            or "rgbw" in fibaro_device.type
+        )
+        supports_dimming = (
+            "levelChange" in fibaro_device.interfaces
+            or supports_color
+            or supports_white_v
         )
-        supports_dimming = "levelChange" in fibaro_device.interfaces
-        supports_white_v = "setW" in fibaro_device.actions
 
         # Configuration can override default capability detection
-        if devconf.get(CONF_DIMMING, supports_dimming):
+        if supports_dimming:
             self._supported_flags |= SUPPORT_BRIGHTNESS
-        if devconf.get(CONF_COLOR, supports_color):
+        if supports_color:
             self._supported_flags |= SUPPORT_COLOR
-        if devconf.get(CONF_WHITE_VALUE, supports_white_v):
+        if supports_white_v:
             self._supported_flags |= SUPPORT_WHITE_VALUE
 
         super().__init__(fibaro_device)
@@ -121,13 +143,11 @@ class FibaroLight(FibaroDevice, Light):
                 # We set it to the target brightness and turn it on
                 self._brightness = scaleto100(target_brightness)
 
-        if self._supported_flags & SUPPORT_COLOR:
-            if (
-                self._reset_color
-                and kwargs.get(ATTR_WHITE_VALUE) is None
-                and kwargs.get(ATTR_HS_COLOR) is None
-                and kwargs.get(ATTR_BRIGHTNESS) is None
-            ):
+        if self._supported_flags & SUPPORT_COLOR and (
+            kwargs.get(ATTR_WHITE_VALUE) is not None
+            or kwargs.get(ATTR_HS_COLOR) is not None
+        ):
+            if self._reset_color:
                 self._color = (100, 0)
 
             # Update based on parameters
@@ -135,14 +155,14 @@ class FibaroLight(FibaroDevice, Light):
             self._color = kwargs.get(ATTR_HS_COLOR, self._color)
             rgb = color_util.color_hs_to_RGB(*self._color)
             self.call_set_color(
-                round(rgb[0] * self._brightness / 100.0),
-                round(rgb[1] * self._brightness / 100.0),
-                round(rgb[2] * self._brightness / 100.0),
-                round(self._white * self._brightness / 100.0),
+                round(rgb[0]),
+                round(rgb[1]),
+                round(rgb[2]),
+                round(self._white),
             )
 
             if self.state == "off":
-                self.set_level(int(self._brightness))
+                self.set_level(min(int(self._brightness), 99))
             return
 
         if self._reset_color:
@@ -150,7 +170,7 @@ class FibaroLight(FibaroDevice, Light):
             self.call_set_color(bri255, bri255, bri255, bri255)
 
         if self._supported_flags & SUPPORT_BRIGHTNESS:
-            self.set_level(int(self._brightness))
+            self.set_level(min(int(self._brightness), 99))
             return
 
         # The simplest case is left for last. No dimming, just switch on
@@ -174,9 +194,28 @@ class FibaroLight(FibaroDevice, Light):
         self.call_turn_off()
 
     @property
-    def is_on(self):
-        """Return true if device is on."""
-        return self.current_binary_state
+    def is_on(self) -> bool | None:
+        """Return true if device is on.
+
+        Dimmable and RGB lights can be on based on different
+        properties, so we need to check here several values.
+
+        JSON for HC2 uses always string, HC3 uses int for integers.
+        """
+        props = self.fibaro_device.properties
+        if self.current_binary_state:
+            return True
+        with suppress(ValueError, TypeError):
+            if "brightness" in props and int(props.brightness) != 0:
+                return True
+        with suppress(ValueError, TypeError):
+            if "currentProgram" in props and int(props.currentProgram) != 0:
+                return True
+        with suppress(ValueError, TypeError):
+            if "currentProgramID" in props and int(props.currentProgramID) != 0:
+                return True
+
+        return False
 
     async def async_update(self):
         """Update the state."""
@@ -206,4 +245,4 @@ class FibaroLight(FibaroDevice, Light):
             if rgbw_list[0] or rgbw_list[1] or rgbw_list[2]:
                 self._color = color_util.color_RGB_to_hs(*rgbw_list[:3])
             if (self._supported_flags & SUPPORT_WHITE_VALUE) and self.brightness != 0:
-                self._white = min(255, max(0, rgbw_list[3] * 100.0 / self._brightness))
+                self._white = min(255, max(0, rgbw_list[3]))
