@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 from datetime import datetime
-import logging
 import struct
 from typing import Any
 
@@ -12,6 +11,7 @@ from homeassistant.components.climate.const import (
     SUPPORT_TARGET_TEMPERATURE,
 )
 from homeassistant.const import (
+    ATTR_TEMPERATURE,
     CONF_NAME,
     CONF_TEMPERATURE_UNIT,
     PRECISION_TENTHS,
@@ -27,25 +27,19 @@ from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from . import get_hub
 from .base_platform import BaseStructPlatform
 from .const import (
-    ATTR_TEMPERATURE,
     CALL_TYPE_REGISTER_HOLDING,
+    CALL_TYPE_WRITE_REGISTER,
     CALL_TYPE_WRITE_REGISTERS,
     CONF_CLIMATES,
     CONF_MAX_TEMP,
     CONF_MIN_TEMP,
     CONF_STEP,
     CONF_TARGET_TEMP,
-    DATA_TYPE_INT16,
-    DATA_TYPE_INT32,
-    DATA_TYPE_INT64,
-    DATA_TYPE_UINT16,
-    DATA_TYPE_UINT32,
-    DATA_TYPE_UINT64,
+    DataType,
 )
 from .modbus import ModbusHub
 
 PARALLEL_UPDATES = 1
-_LOGGER = logging.getLogger(__name__)
 
 
 async def async_setup_platform(
@@ -109,18 +103,16 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
 
     async def async_set_temperature(self, **kwargs: Any) -> None:
         """Set new target temperature."""
-        if ATTR_TEMPERATURE not in kwargs:
-            return
         target_temperature = (
             float(kwargs[ATTR_TEMPERATURE]) - self._offset
         ) / self._scale
         if self._data_type in (
-            DATA_TYPE_INT16,
-            DATA_TYPE_INT32,
-            DATA_TYPE_INT64,
-            DATA_TYPE_UINT16,
-            DATA_TYPE_UINT32,
-            DATA_TYPE_UINT64,
+            DataType.INT16,
+            DataType.INT32,
+            DataType.INT64,
+            DataType.UINT16,
+            DataType.UINT32,
+            DataType.UINT64,
         ):
             target_temperature = int(target_temperature)
         as_bytes = struct.pack(self._structure, target_temperature)
@@ -129,12 +121,24 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
             for i in range(0, len(as_bytes), 2)
         ]
         registers = self._swap_registers(raw_regs)
-        result = await self._hub.async_pymodbus_call(
-            self._slave,
-            self._target_temperature_register,
-            registers,
-            CALL_TYPE_WRITE_REGISTERS,
-        )
+
+        if self._data_type in (
+            DataType.INT16,
+            DataType.UINT16,
+        ):
+            result = await self._hub.async_pymodbus_call(
+                self._slave,
+                self._target_temperature_register,
+                int(float(registers[0])),
+                CALL_TYPE_WRITE_REGISTER,
+            )
+        else:
+            result = await self._hub.async_pymodbus_call(
+                self._slave,
+                self._target_temperature_register,
+                [int(float(i)) for i in registers],
+                CALL_TYPE_WRITE_REGISTERS,
+            )
         self._attr_available = result is not None
         await self.async_update()
 
@@ -173,7 +177,8 @@ class ModbusThermostat(BaseStructPlatform, RestoreEntity, ClimateEntity):
 
         self._lazy_errors = self._lazy_error_count
         self._value = self.unpack_structure_result(result.registers)
-        self._attr_available = True
         if not self._value:
+            self._attr_available = False
             return None
+        self._attr_available = True
         return float(self._value)
