@@ -98,9 +98,7 @@ class PlexFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         self.client_id = None
         self._manual = False
 
-    async def async_step_user(
-        self, user_input=None, errors=None
-    ):  # pylint: disable=arguments-differ
+    async def async_step_user(self, user_input=None, errors=None):
         """Handle a flow initialized by the user."""
         if user_input is not None:
             return await self.async_step_plex_website_auth()
@@ -246,26 +244,26 @@ class PlexFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         """Use selected Plex server."""
         config = dict(self.current_login)
         if user_input is not None:
-            config[CONF_SERVER] = user_input[CONF_SERVER]
+            config[CONF_SERVER_IDENTIFIER] = user_input[CONF_SERVER_IDENTIFIER]
             return await self.async_step_server_validate(config)
 
         configured = configured_servers(self.hass)
-        available_servers = [
-            name
-            for (name, server_id) in self.available_servers
+        available_servers = {
+            server_id: f"{name} ({owner})" if owner else name
+            for (name, server_id, owner) in self.available_servers
             if server_id not in configured
-        ]
+        }
 
         if not available_servers:
             return self.async_abort(reason="all_configured")
         if len(available_servers) == 1:
-            config[CONF_SERVER] = available_servers[0]
+            config[CONF_SERVER_IDENTIFIER] = next(iter(available_servers))
             return await self.async_step_server_validate(config)
 
         return self.async_show_form(
             step_id="select_server",
             data_schema=vol.Schema(
-                {vol.Required(CONF_SERVER): vol.In(available_servers)}
+                {vol.Required(CONF_SERVER_IDENTIFIER): vol.In(available_servers)}
             ),
             errors={},
         )
@@ -380,6 +378,7 @@ class PlexOptionsFlowHandler(config_entries.OptionsFlow):
                 for user in plex_server.option_monitored_users
                 if plex_server.option_monitored_users[user]["enabled"]
             }
+            default_accounts.intersection_update(plex_server.accounts)
             for user in plex_server.accounts:
                 if user not in known_accounts:
                     available_accounts[user] += " [New]"
