@@ -1,4 +1,6 @@
 """Support for departure information for Rhein-Main public transport."""
+from __future__ import annotations
+
 import asyncio
 from datetime import timedelta
 import logging
@@ -10,11 +12,13 @@ from RMVtransport.rmvtransport import (
 )
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA
+from homeassistant.components.sensor import PLATFORM_SCHEMA, SensorEntity
 from homeassistant.const import ATTR_ATTRIBUTION, CONF_NAME, CONF_TIMEOUT, TIME_MINUTES
+from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import PlatformNotReady
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util import Throttle
 
 _LOGGER = logging.getLogger(__name__)
@@ -75,7 +79,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+async def async_setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the RMV departure sensor."""
     timeout = config.get(CONF_TIMEOUT)
 
@@ -91,7 +100,7 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
             next_departure.get(CONF_NAME),
             timeout,
         )
-        for next_departure in config.get(CONF_NEXT_DEPARTURE)
+        for next_departure in config[CONF_NEXT_DEPARTURE]
     ]
 
     tasks = [sensor.async_update() for sensor in sensors]
@@ -104,7 +113,7 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
     async_add_entities(sensors)
 
 
-class RMVDepartureSensor(Entity):
+class RMVDepartureSensor(SensorEntity):
     """Implementation of an RMV departure sensor."""
 
     def __init__(
@@ -146,12 +155,12 @@ class RMVDepartureSensor(Entity):
         return self._state is not None
 
     @property
-    def state(self):
+    def native_value(self):
         """Return the next departure time."""
         return self._state
 
     @property
-    def state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes."""
         try:
             return {
@@ -172,7 +181,7 @@ class RMVDepartureSensor(Entity):
         return self._icon
 
     @property
-    def unit_of_measurement(self):
+    def native_unit_of_measurement(self):
         """Return the unit this state is expressed in."""
         return TIME_MINUTES
 
@@ -263,7 +272,7 @@ class RMVDepartureData:
             elif journey["minutes"] < self._time_offset:
                 continue
 
-            for attr in ["direction", "departure_time", "product", "minutes"]:
+            for attr in ("direction", "departure_time", "product", "minutes"):
                 _nextdep[attr] = journey.get(attr, "")
 
             _nextdep["line"] = journey.get("number", "")
