@@ -1,7 +1,8 @@
 """Base class for all Subaru Entities."""
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
-from .const import DOMAIN, ICONS, MANUFACTURER, VEHICLE_NAME, VEHICLE_VIN
+from .const import DOMAIN, MANUFACTURER, VEHICLE_NAME, VEHICLE_VIN
 
 
 class SubaruEntity(CoordinatorEntity):
@@ -25,15 +26,10 @@ class SubaruEntity(CoordinatorEntity):
         return f"{self.vin}_{self.entity_type}"
 
     @property
-    def icon(self):
-        """Return the icon of the sensor."""
-        return ICONS.get(self.entity_type)
-
-    @property
-    def device_info(self):
+    def device_info(self) -> DeviceInfo:
         """Return the device_info of the device."""
-        return {
-            "identifiers": {(DOMAIN, self.vin)},
-            "name": self.car_name,
-            "manufacturer": MANUFACTURER,
-        }
+        return DeviceInfo(
+            identifiers={(DOMAIN, self.vin)},
+            manufacturer=MANUFACTURER,
+            name=self.car_name,
+        )
