@@ -4,6 +4,7 @@ from __future__ import annotations
 import asyncio
 from collections import defaultdict
 from collections.abc import Callable
+from typing import Any
 
 from async_timeout import timeout
 from zwave_js_server.client import Client as ZwaveClient
@@ -12,9 +13,11 @@ from zwave_js_server.model.node import Node as ZwaveNode
 from zwave_js_server.model.notification import (
     EntryControlNotification,
     NotificationNotification,
+    PowerLevelNotification,
 )
 from zwave_js_server.model.value import Value, ValueNotification
 
+from homeassistant.components.button import DOMAIN as BUTTON_DOMAIN
 from homeassistant.components.sensor import DOMAIN as SENSOR_DOMAIN
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
@@ -40,6 +43,7 @@ from homeassistant.helpers.typing import ConfigType
 from .addon import AddonError, AddonManager, AddonState, get_addon_manager
 from .api import async_register_api
 from .const import (
+    ATTR_ACKNOWLEDGED_FRAMES,
     ATTR_COMMAND_CLASS,
     ATTR_COMMAND_CLASS_NAME,
     ATTR_DATA_TYPE,
@@ -56,6 +60,8 @@ from .const import (
     ATTR_PROPERTY_KEY,
     ATTR_PROPERTY_KEY_NAME,
     ATTR_PROPERTY_NAME,
+    ATTR_STATUS,
+    ATTR_TEST_NODE_ID,
     ATTR_TYPE,
     ATTR_VALUE,
     ATTR_VALUE_RAW,
@@ -88,7 +94,13 @@ from .discovery import (
     async_discover_node_values,
     async_discover_single_value,
 )
-from .helpers import async_enable_statistics, get_device_id, get_unique_id
+from .helpers import (
+    async_enable_statistics,
+    get_device_id,
+    get_device_id_ext,
+    get_unique_id,
+    get_valueless_base_unique_id,
+)
 from .migrate import async_migrate_discovered_value
 from .services import ZWaveServices
 
@@ -102,6 +114,11 @@ DATA_INVALID_SERVER_VERSION_LOGGED = "invalid_server_version_logged"
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the Z-Wave JS component."""
     hass.data[DOMAIN] = {}
+    for entry in hass.config_entries.async_entries(DOMAIN):
+        if not isinstance(entry.unique_id, str):
+            hass.config_entries.async_update_entry(
+                entry, unique_id=str(entry.unique_id)
+            )
     return True
 
 
@@ -116,17 +133,27 @@ def register_node_in_dev_reg(
 ) -> device_registry.DeviceEntry:
     """Register node in dev reg."""
     device_id = get_device_id(client, node)
-    # If a device already exists but it doesn't match the new node, it means the node
-    # was replaced with a different device and the device needs to be removeed so the
-    # new device can be created. Otherwise if the device exists and the node is the same,
-    # the node was replaced with the same device model and we can reuse the device.
-    if (device := dev_reg.async_get_device({device_id})) and (
-        device.model != node.device_config.label
-        or device.manufacturer != node.device_config.manufacturer
+    device_id_ext = get_device_id_ext(client, node)
+    device = dev_reg.async_get_device({device_id})
+
+    # Replace the device if it can be determined that this node is not the
+    # same product as it was previously.
+    if (
+        device_id_ext
+        and device
+        and len(device.identifiers) == 2
+        and device_id_ext not in device.identifiers
     ):
         remove_device_func(device)
+        device = None
+
+    if device_id_ext:
+        ids = {device_id, device_id_ext}
+    else:
+        ids = {device_id}
+
     params = {
-        ATTR_IDENTIFIERS: {device_id},
+        ATTR_IDENTIFIERS: ids,
         ATTR_SW_VERSION: node.firmware_version,
         ATTR_NAME: node.name
         or node.device_config.description
@@ -156,11 +183,19 @@ async def async_setup_entry(  # noqa: C901
     entry_hass_data: dict = hass.data[DOMAIN].setdefault(entry.entry_id, {})
 
     entry_hass_data[DATA_CLIENT] = client
-    entry_hass_data[DATA_PLATFORM_SETUP] = {}
+    platform_setup_tasks = entry_hass_data[DATA_PLATFORM_SETUP] = {}
 
     registered_unique_ids: dict[str, dict[str, set[str]]] = defaultdict(dict)
     discovered_value_ids: dict[str, set[str]] = defaultdict(set)
 
+    async def async_setup_platform(platform: str) -> None:
+        """Set up platform if needed."""
+        if platform not in platform_setup_tasks:
+            platform_setup_tasks[platform] = hass.async_create_task(
+                hass.config_entries.async_forward_entry_setup(entry, platform)
+            )
+        await platform_setup_tasks[platform]
+
     @callback
     def remove_device(device: device_registry.DeviceEntry) -> None:
         """Remove device from registry."""
@@ -187,13 +222,8 @@ async def async_setup_entry(  # noqa: C901
             disc_info,
         )
 
-        platform_setup_tasks = entry_hass_data[DATA_PLATFORM_SETUP]
         platform = disc_info.platform
-        if platform not in platform_setup_tasks:
-            platform_setup_tasks[platform] = hass.async_create_task(
-                hass.config_entries.async_forward_entry_setup(entry, platform)
-            )
-        await platform_setup_tasks[platform]
+        await async_setup_platform(platform)
 
         LOGGER.debug("Discovered entity: %s", disc_info)
         async_dispatcher_send(
@@ -260,33 +290,23 @@ async def async_setup_entry(  # noqa: C901
             )
         )
         # add listener for stateless node notification events
-        entry.async_on_unload(
-            node.on(
-                "notification",
-                lambda event: async_on_notification(event["notification"]),
-            )
-        )
+        entry.async_on_unload(node.on("notification", async_on_notification))
 
     async def async_on_node_added(node: ZwaveNode) -> None:
         """Handle node added event."""
-        platform_setup_tasks = entry_hass_data[DATA_PLATFORM_SETUP]
-
-        # We need to set up the sensor platform if it hasn't already been setup in
-        # order to create the node status sensor
-        if SENSOR_DOMAIN not in platform_setup_tasks:
-            platform_setup_tasks[SENSOR_DOMAIN] = hass.async_create_task(
-                hass.config_entries.async_forward_entry_setup(entry, SENSOR_DOMAIN)
+        # No need for a ping button or node status sensor for controller nodes
+        if not node.is_controller_node:
+            # Create a node status sensor for each device
+            await async_setup_platform(SENSOR_DOMAIN)
+            async_dispatcher_send(
+                hass, f"{DOMAIN}_{entry.entry_id}_add_node_status_sensor", node
             )
 
-        # This guard ensures that concurrent runs of this function all await the
-        # platform setup task
-        if not platform_setup_tasks[SENSOR_DOMAIN].done():
-            await platform_setup_tasks[SENSOR_DOMAIN]
-
-        # Create a node status sensor for each device
-        async_dispatcher_send(
-            hass, f"{DOMAIN}_{entry.entry_id}_add_node_status_sensor", node
-        )
+            # Create a ping button for each device
+            await async_setup_platform(BUTTON_DOMAIN)
+            async_dispatcher_send(
+                hass, f"{DOMAIN}_{entry.entry_id}_add_ping_button_entity", node
+            )
 
         # we only want to run discovery when the node has reached ready state,
         # otherwise we'll have all kinds of missing info issues.
@@ -338,7 +358,14 @@ async def async_setup_entry(  # noqa: C901
         device = dev_reg.async_get_device({dev_id})
         # We assert because we know the device exists
         assert device
-        if not replaced:
+        if replaced:
+            discovered_value_ids.pop(device.id, None)
+
+            async_dispatcher_send(
+                hass,
+                f"{DOMAIN}_{get_valueless_base_unique_id(client, node)}_remove_entity",
+            )
+        else:
             remove_device(device)
 
     @callback
@@ -371,10 +398,14 @@ async def async_setup_entry(  # noqa: C901
         )
 
     @callback
-    def async_on_notification(
-        notification: EntryControlNotification | NotificationNotification,
-    ) -> None:
+    def async_on_notification(event: dict[str, Any]) -> None:
         """Relay stateless notification events from Z-Wave nodes to hass."""
+        if "notification" not in event:
+            LOGGER.info("Unknown notification: %s", event)
+            return
+        notification: EntryControlNotification | NotificationNotification | PowerLevelNotification = event[
+            "notification"
+        ]
         device = dev_reg.async_get_device({get_device_id(client, notification.node)})
         # We assert because we know the device exists
         assert device
@@ -395,7 +426,7 @@ async def async_setup_entry(  # noqa: C901
                     ATTR_EVENT_DATA: notification.event_data,
                 }
             )
-        else:
+        elif isinstance(notification, NotificationNotification):
             event_data.update(
                 {
                     ATTR_COMMAND_CLASS_NAME: "Notification",
@@ -406,6 +437,17 @@ async def async_setup_entry(  # noqa: C901
                     ATTR_PARAMETERS: notification.parameters,
                 }
             )
+        elif isinstance(notification, PowerLevelNotification):
+            event_data.update(
+                {
+                    ATTR_COMMAND_CLASS_NAME: "Power Level",
+                    ATTR_TEST_NODE_ID: notification.test_node_id,
+                    ATTR_STATUS: notification.status,
+                    ATTR_ACKNOWLEDGED_FRAMES: notification.acknowledged_frames,
+                }
+            )
+        else:
+            raise TypeError(f"Unhandled notification type: {notification}")
 
         hass.bus.async_fire(ZWAVE_JS_NOTIFICATION_EVENT, event_data)
 
@@ -424,9 +466,7 @@ async def async_setup_entry(  # noqa: C901
         # We assert because we know the device exists
         assert device
 
-        unique_id = get_unique_id(
-            client.driver.controller.home_id, disc_info.primary_value.value_id
-        )
+        unique_id = get_unique_id(client, disc_info.primary_value.value_id)
         entity_id = ent_reg.async_get_entity_id(disc_info.platform, DOMAIN, unique_id)
 
         raw_value = value_ = value.value
